package dtu.library.persistency;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertSame;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import dtu.library.app.Address;
import dtu.library.app.Book;
import dtu.library.app.Cd;
import dtu.library.app.LibraryApp;
import dtu.library.app.Medium;
import dtu.library.app.User;

public class TestUseOfPersistencyLayer {

	@Before
	public void setUp() {
		PersistencyLayer.clearDatabase(); // Fresh database state for each test
	}

	@Test
	public void testAddUser() throws Exception {

		PersistencyLayer pl = new PersistencyLayer();

		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
		User user = new User("cpr-number", "Some Name", "a@b.dk", address);

		pl.createUser(user);

		PersistencyLayer pl1 = new PersistencyLayer();

		User user1 = (User) pl1.readUser(user.getCprNumber());

		Utilities.compareUsers(user, user1);
	}

//	@Test
//	public void testAddTwoUsers() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//		Address address = new Address("Kongevejen", 2121, "Kbenhavn");
//		User expected1 = new User("cpr-number2", "Some Name2", "a@b.dk",
//				address);
//
//		pl.createUser(expected1);
//
//		address = new Address("Kongevejen 2", 2121, "Kbenhavn");
//		User expected2 = new User("cpr-number", "Some Name", "a@b.dk", address);
//
//		pl.createUser(expected2);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		User actual = (User) pl1.readUser(expected2.getCprNumber());
//
//		Utilities.compareUsers(expected2, actual);
//
//		actual = (User) pl1.readUser(expected1.getCprNumber());
//		Utilities.compareUsers(expected1, actual);
//	}
//
//	@Test
//	public void testSearcUserByCprNumberNotFound() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//
//		pl.createUser(user);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		User user1 = (User) pl1.readUser(user.getCprNumber() + "1");
//
//		assertNull(user1);
//	}
//
//	@Test
//	public void testUpdateUser() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//
//		pl.createUser(user);
//
//		pl = new PersistencyLayer();
//
//		User user1 = (User) pl.readUser(user.getCprNumber());
//
//		user1.setName("New Name");
//
//		pl.updateUser(user1);
//
//		pl = new PersistencyLayer();
//
//		User user2 = (User) pl.readUser(user.getCprNumber());
//
//		assertEquals("New Name", user2.getName());
//
//		Utilities.compareUsers(user1, user2);
//	}
//
//	@Test
//	public void testAddBook() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Book book = new Book("Som01", "Software Engineering", "Ian Sommerville");
//
//		pl.createMedium(book);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		Book book1 = (Book) pl1.readMedium(book.getSignature());
//
//		Utilities.compareMedium(book, book1);
//	}
//
//	@Test
//	public void testUpdateBook() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Book book = new Book("Som01", "Software Engineering", "Ian Wrong");
//
//		pl.createMedium(book);
//
//		book.setAuthor("Ian Sommerville");
//
//		pl.updateMedium(book);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		Book book1 = (Book) pl1.readMedium(book.getSignature());
//
//		Utilities.compareMedium(book, book1);
//	}
//
//	@Test
//	public void testAddCd() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Cd book = new Cd("Cd01", "Some title", "Some author");
//
//		pl.createMedium(book);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		Cd book1 = (Cd) pl1.readMedium(book.getSignature());
//
//		Utilities.compareMedium(book, book1);
//	}
//
//	@Test
//	public void testUpdateCd() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Cd book = new Cd("Cd01", "Wrong title", "Some Author");
//
//		pl.createMedium(book);
//
//		book.setTitle("Some title");
//
//		pl.updateMedium(book);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		Cd book1 = (Cd) pl1.readMedium(book.getSignature());
//
//		Utilities.compareMedium(book, book1);
//	}
//
//	@Test
//	public void testSearchInEmptyDatabase() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		User po = pl.readUser("some key");
//
//		assertNull(po);
//
//	}
//
//	@Test
//	public void testAddUserWithBorrowedMedia() throws Exception {
//
//		LibraryApp libApp = new LibraryApp();
//
//		PersistencyLayer pl = new PersistencyLayer();
//		libApp.setPersistencyLayer(pl);
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//		user.setLibraryApp(libApp);
//
//		Book b1 = new Book("b01", "some book title", "some book author");
//		b1.setLibraryApp(libApp);
//		Cd c1 = new Cd("c01", "some cd title", "some cd author");
//		c1.setLibraryApp(libApp);
//
//		pl.createUser(user);
//		pl.createMedium(b1);
//		pl.createMedium(c1);
//
//		user.borrowMedium(b1);
//		user.borrowMedium(c1);
//
//		pl.updateMedium(b1);
//		pl.updateMedium(c1);
//		pl.updateUser(user);
//
//		libApp = new LibraryApp();
//		PersistencyLayer pl1 = new PersistencyLayer();
//		libApp.setPersistencyLayer(pl1);
//
//		User user1 = (User) pl1.readUser(user.getCprNumber());
//
//		Utilities.compareUsers(user, user1);
//	}
//
//	@Test
//	public void testAddUserWithBorrowedMedia2() throws Exception {
//
//		LibraryApp libApp = new LibraryApp();
//
//		PersistencyLayer pl = new PersistencyLayer();
//		libApp.setPersistencyLayer(pl);
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//		user.setLibraryApp(libApp);
//
//		Book b1 = new Book("b01", "some book title", "some book author");
//		b1.setLibraryApp(libApp);
//		Cd c1 = new Cd("c01", "some cd title", "some cd author");
//		c1.setLibraryApp(libApp);
//
//		pl.createMedium(b1);
//		pl.createMedium(c1);
//		pl.createUser(user);
//
//		user.borrowMedium(b1);
//		user.borrowMedium(c1);
//
//		pl.updateUser(user);
//		pl.updateMedium(c1);
//		pl.updateMedium(b1);
//
//		libApp = new LibraryApp();
//		PersistencyLayer pl1 = new PersistencyLayer();
//		libApp.setPersistencyLayer(pl1);
//
//		User user1 = pl1.readUser(user.getCprNumber());
//
//		Utilities.compareUsers(user, user1);
//	}
//
//	@Test
//	public void testSearchUserTwoTimes() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//
//		pl.createUser(user);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		User user1 = pl1.readUser(user.getCprNumber());
//		User user2 = pl1.readUser(user.getCprNumber());
//
//		assertSame(user1, user2);
//	}
//
//	@Test
//	public void testSearchBookTwoTimes() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Medium user = new Book("b01", "Some title", "Some author");
//		pl.createMedium(user);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		Medium user1 = pl1.readMedium(user.getSignature());
//		Medium user2 = pl1.readMedium(user.getSignature());
//
//		assertSame(user1, user2);
//	}
//
//	@Test
//	public void testAddAndSearchTheSame() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//
//		pl.createUser(user);
//
//		User user1 = (User) pl.readUser(user.getCprNumber());
//
//		assertSame(user, user1);
//	}
//
//	@Test
//	public void testDeleteUser1() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//
//		pl.createUser(user);
//		User user1 = (User) pl.readUser(user.getCprNumber());
//		assertSame(user, user1);
//
//		pl.deleteUser(user.getCprNumber());
//
//		assertNull(pl.readUser(user.getCprNumber()));
//	}
//
//	@Test
//	public void testDeleteUser2() throws Exception {
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Book user = new Book("b01", "Some title", "Some author");
//		pl.createMedium(user);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		Medium user1 = pl1.readMedium(user.getSignature());
//
//		pl1.deleteMedium(user1.getSignature());
//
//		pl1 = new PersistencyLayer();
//
//		assertNull(pl1.readMedium(user.getSignature()));
//	}
//
//	@Test
//	public void testGetUsers() throws Exception {
//
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Address address = new Address("Kongevejen", 2120, "Kbenhavn");
//		User user = new User("cpr-number", "Some Name", "a@b.dk", address);
//
//		pl.createUser(user);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		List<User> list = pl1.getUsers();
//
//		assertEquals(1, list.size());
//
//		Utilities.compareUsers(user, list.get(0));
//	}
//
//	@Test
//	public void testGetMedia() throws Exception {
//
//		PersistencyLayer pl = new PersistencyLayer();
//
//		Medium user = new Book("B01", "Some title", "Some author");
//		pl.createMedium(user);
//
//		PersistencyLayer pl1 = new PersistencyLayer();
//
//		List<Medium> list = pl1.getMedia();
//
//		assertEquals(1, list.size());
//
//		Utilities.compareMedium(user, list.get(0));
//	}

}